#!/usr/bin/python

import sys
import unittest
from urlparse import urlparse

import piston_mini_client
from mock import patch
from piston_mini_client.failhandlers import NoneFailHandler
from rnrclient import RatingsAndReviewsAPI, ReviewRequest

class TestRnRClient(unittest.TestCase):

    def setUp(self):
        self.rnrclient = RatingsAndReviewsAPI("http://testme/review/api/1.0")
        self.rnrclient._get = self._monkey_patched_get

    def _monkey_patched_get(self, url, scheme=None):
        self.hit_url = url
        return ("200 ok", "[]")

    def test_get_review(self):
        # simple case, just pkgname
        self.rnrclient.get_reviews(packagename="foo")
        self.assertEqual(self.hit_url, "any/any/any/any/foo/page/1/")
        # pkgname with language, origin and distroseries
        self.rnrclient.get_reviews(language="zh_CN", origin="ubuntu", 
                                   distroseries="lucid", packagename="foo")
        self.assertEqual(self.hit_url, "zh_CN/ubuntu/lucid/any/foo/page/1/")
        # pkgname with version
        self.rnrclient.get_reviews(version="1.0", packagename="foo")
        self.assertEqual(self.hit_url, "any/any/any/1.0/foo/page/1/")
        # pkgname with appname and space
        self.rnrclient.get_reviews(language="en", packagename="foo",
                                   appname="Foo Bar")
        self.assertEqual(self.hit_url, "en/any/any/any/foo%3BFoo+Bar/page/1/")
        # complicated appname
        self.rnrclient.get_reviews(packagename="foo", appname="Foo!;-+*;)")
        self.assertEqual(self.hit_url, 
            "any/any/any/any/foo%3BFoo%21%3B-%2B%2A%3B%29/page/1/")
        # invalid pkgname
        self.assertRaises(piston_mini_client.validators.ValidationException,
                          self.rnrclient.get_reviews,
                          packagename="foo;invalid")
        # invalid distroseries
        kwargs = {"distroseries" : "lucid;invalid-chars", 
                  "packagename" : "foo" }
        self.assertRaises(piston_mini_client.validators.ValidationException,
                          self.rnrclient.get_reviews, kwargs)
        # valid (but complicated) ppa origin
        kwargs = {"language" : "en", 
                  "origin" : "ppa:a.bono-wicked/ppa+fun-0.96,1",
                  "packagename" : "foo" }
        self.rnrclient.get_reviews(**kwargs)
        self.assertEqual(self.hit_url,
            "en/ppa:a.bono-wicked/ppa+fun-0.96,1/any/any/foo/page/1/")

    def test_get_review_stats(self):
        self.rnrclient.review_stats()
        self.assertEqual(self.hit_url, "review-stats/")
        self.rnrclient.review_stats(days=4)
        self.assertEqual(self.hit_url, "review-stats/updates-last-7-days/")
        self.rnrclient.review_stats(days=14)
        self.assertEqual(self.hit_url, "review-stats/")
        self.rnrclient.review_stats(days=1)
        self.assertEqual(self.hit_url, "review-stats/updates-last-1-days/")
        self.rnrclient.review_stats(days=3)
        self.assertEqual(self.hit_url, "review-stats/updates-last-3-days/")
        self.rnrclient.review_stats(days=2)
        self.assertEqual(self.hit_url, "review-stats/updates-last-3-days/")

    @patch('httplib2.Http.request')
    def test_schemes(self, mock_request):
        mock_request.return_value = {}, '"uh?"'
        def called_scheme():
            return urlparse(mock_request.call_args[0][0]).scheme
        for service_root_scheme in ['http', 'https']:
            service_root = service_root_scheme + "://testme/review/api/1.0"
            rnrclient = RatingsAndReviewsAPI(service_root)
            rnrclient.fail_handler = NoneFailHandler
            rnrclient.server_status()
            self.assertEquals('http', called_scheme())
            rnrclient.review_stats()
            self.assertEquals('http', called_scheme())
            rnrclient.get_reviews(packagename="foo")
            self.assertEquals('http', called_scheme())
            review = ReviewRequest(package_name='foo', summary='bar',
                version='1.0', review_text='baz', rating=4, language='en',
                origin='troz', distroseries='eegad', arch_tag='zot')
            rnrclient.submit_review(review=review)
            self.assertEquals('https', called_scheme())
            rnrclient.flag_review(review_id=1, reason='foo', text='bar')
            self.assertEquals('https', called_scheme())
            rnrclient.submit_usefulness(review_id=1, useful='True')
            self.assertEquals('https', called_scheme())
        

if __name__ == "__main__":
    unittest.main()
